
var _ = require("underscore")
	, pad = require('pad')
	, SubField = require('./subfield');

function Field (name) {
	this.options = {};

	this.set('name',  pad(3, name, '0'));
	
	if(this.isControlField()) {
		if(! _.isUndefined(arguments[1])) {
			this.set('value', arguments[1]);		
		}
	}else {
		this.set('subfields', new Array());
	}
};

Field.prototype.set =
Field.prototype.get = function (key, value) {
  if (arguments.length == 1)
    return this.options[key];
  this.options[key] = value;
  return this;
};

Field.prototype.isControlField = function() {
	return (parseFloat(this.get('name')) <= 8);
}

Field.prototype.name = function() {	

	if (arguments.length == 0) {
		return this.get('name');
	}

	this.set('name', arguments[0]);
	return this;
}

Field.prototype.value = function() {
	if(! this.isControlField()) { return false; }
	
	if (arguments.length == 0) {
		return this.get('value');
	}

	this.set('value', arguments[0]);
	return this;
}

Field.prototype.subfields = function() {
	if(this.isControlField()) { return false; }
	return this.get('subfields');
}

Field.prototype.ind1 = function() {

	if (arguments.length == 0) {
		return this.get('ind1');
	}

	if(this.isControlField()) { return this; }
	this.set('ind1', arguments[0]);
	return this;
}

Field.prototype.ind2 = function() {
	
	if (arguments.length == 0) {
		return this.get('ind2');
	}

	if(this.isControlField()) { return this; }
	this.set('ind2', arguments[0]);
	return this;
}

Field.prototype.subfield = function() {
	
	if(this.isControlField()) { return this; }

	if (arguments.length == 1) {
		var desiredSubFieldName = arguments[0];
		var subfields = _.filter(this.subfields(), function(subfield) {
			return (subfield.name() === desiredSubFieldName);
		});

		if(_.size(subfields) == 1) {
			return _.first(subfields);
		}else {
			return subfields;
		}
	}

	this.subfields().push(new SubField(arguments[0], arguments[1]));
	return this;
}

/**
 * deleteSub()
 *
 * @param {Mixed} obj
 * @api public
 */
Field.prototype.deleteSub = function(subNames) {

	var names = new Array();
	if(! _.isArray(subNames)) {
		names.push(subNames);
	}else {
		names = subNames;
	}

	var newSubFields = _.filter(this.subfields(), function(subfield){
		if(! _.include(names, subfield.name() )) return subfield;
	});

	this.set('subfields', newSubFields);
	return this;
}

module.exports = exports = Field;