var fs = require('fs')
	, _ = require("underscore")
	, S = require('string')
	, util = require('util')
	, Field = require('./field')
	, Marc = require('./index')
	, util = require('util');

function Reader () {}

Reader.prototype.read = function() {
	
	var source = arguments[0];

	if(_.isObject(source)) {
		// Read as JSON-HASH
		return this._parseFromJson(source);

	}else if(_.isString(source)) {
		// Read as file
		try {
			var marc = fs.readFileSync(source);
			var record = this._parse(marc);
			record.marc(marc.toString());
			var leader = S(marc.toString()).left(24).s;
			record.info().record_status(leader[5]);
			record.info().type_of_record(leader[6]);
			record.info().bibliographic_level(leader[7]);
			record.info().type_of_control(leader[8]);
			record.info().character_coding_scheme(leader[9]);
			record.info().indicator_count(leader[10]);
			record.info().subfield_code_count(leader[11]);
			record.info().encoding_level(leader[17]);
			record.info().descriptive_cataloging_form(leader[18]);
			record.info().multipart_resource_record_level(leader[19]);
			record.info().length_of_length_field(leader[20]);
			record.info().length_of_starting_char_position(leader[21]);
			record.info().length_of_implementation_defined_portion(leader[22]);


			return record;
		}catch(error) {
			return false;
		}
	}
}

Reader.prototype._parse = function(buf) {
	
	var record = new Marc.Record();
	
	var baseaddress = buf.slice(12, 17).toString();
	var values = buf.slice(parseFloat(baseaddress));
	var data = buf.slice(0, parseFloat(baseaddress) - 1);
	
	for(var i = 24; i < data.length; i = i + 12 ) {

		var field = data.slice(i, i + 3);
		var characterCount = parseFloat(data.slice(i + 3, i + 7));
		var where = parseFloat(data.slice(i + 7, i + 12));
		
		var isControlField = false;
		
		if(parseFloat(field) <= 8) {
			isControlField = true;
		}

		var fieldValue = values.slice(where, where + characterCount).toString();
		fieldValue = fieldValue.substr(0, fieldValue.length - 1);

		if(isControlField) {
			record.fields().push(new Field(field.toString(), fieldValue.toString()));
		}else {
			
			
			var buf = new Buffer(fieldValue);
			var fieldValues = buf.toString('hex').split(/1f/);
			var indicators = new Buffer(_.first(fieldValues), 'hex').toString();
			
			var dataField = new Field(field.toString());
			dataField
			.ind1(indicators[0])
			.ind2(indicators[1]);

			_.each(_.rest(fieldValues, 1), function(f) {
				var val = new Buffer(f, 'hex').toString();
				dataField.subfield(val[0], val.substr(1));
			});				

			record.fields().push(dataField);
		}

		if( i >= parseFloat(baseaddress) ) break;
    }

    return record;
}

Reader.prototype._parseFromJson = function(json) {

	var record = new Marc.Record();

	_.each(json.fields, function(field) {
		
		if(_.size(field) == 2) {
			// Control Field
			record.addField(field[0], field[1]);
		}else {
			// Data Field
			var newfield = record.addField(field[0])
								.ind1(field[1])
								.ind2(field[2]);
			
			_.each(field[3], function(subfield) {
				newfield.subfield(subfield[0], subfield[1]);
			});
		}
	});

	//console.log(util.inspect(record, true, null, false))
	
	var leader = json.leader;
	record.info().record_status(leader[5]);
	record.info().type_of_record(leader[6]);
	record.info().bibliographic_level(leader[7]);
	record.info().type_of_control(leader[8]);
	record.info().character_coding_scheme(leader[9]);
	record.info().indicator_count(leader[10]);
	record.info().subfield_code_count(leader[11]);
	record.info().encoding_level(leader[17]);
	record.info().descriptive_cataloging_form(leader[18]);
	record.info().multipart_resource_record_level(leader[19]);
	record.info().length_of_length_field(leader[20]);
	record.info().length_of_starting_char_position(leader[21]);
	record.info().length_of_implementation_defined_portion(leader[22]);

	return record;
}

module.exports = exports = Reader;