
var _ = require('underscore')
	, Leader = require('./leader')
	, pad = require('pad')
	, Info = require('./info')
	, Field = require('./field')
	, S = require('string')
	, _us = new Buffer('1f', 'hex').toString()
	, _rs = new Buffer('1e', 'hex').toString()
	, _gs = new Buffer('1d', 'hex').toString();

function Record () {
	
	if(! _.isUndefined(arguments[0])) {
		this.options = arguments[0];
	}else {
		this.options = {};
	}

	this.set('fields', new Array());
	this.set('leader', new Leader())
	this.set('info', new Info());;
	this.set('json', null);
	this.set('marc', null);
	this.set('modified', false);
};

Record.prototype.set =
Record.prototype.get = function (key, value) {
  if (arguments.length == 1)
    return this.options[key];
  this.options[key] = value;
  return this;
};

Record.prototype.info = function() {
	return this.get('info');
}

Record.prototype.fields = function() {
	return this.get('fields');
}

Record.prototype.modified = function() {
	if(arguments.length == 1) {
		this.set('modified', arguments[0]);
		return this;
	}

	return this.get('modified');
}

Record.prototype.field = function(name) {

	var field = _.find(this.get('fields'), function(field) {
		return (parseFloat(field.name()) == parseFloat(name));
	});

	if(_.isUndefined(field)) {
		if(arguments.length == 2) {
			return new Array();	
		}else {
			return false;	
		}
	}

	if(! _.isUndefined(field) && arguments.length == 2) {
		var args = arguments;
		var subfields = _.filter(field.subfields(), function(subfield) {
			return (subfield.name() === args[1]);
		});
		
		if(_.isUndefined(subfields)) {
			return new Array();
		}else {
			return subfields;
		}
	}

	return field;
}

Record.prototype.hasField = function(name) {
	var field = _.find(this.get('fields'), function(field) {
		return (parseFloat(field.get('name')) == parseFloat(name));
	});

	return (! _.isUndefined(field));
}

Record.prototype.toJson = function() {
	
	if(! this.modified() && ! _.isNull(this.get('json'))) {
		return this.get('json');
	}

	// Create the JSON format
	var jsonHash = {
		type: "marc-hash",
		version: [1, 0],
		leader: this.leader(),
		fields: new Array()
	};
	
	_.each(this.get('fields'), function(field) {

		if(parseFloat(field.name()) > 8 ) {
			
			var tempField = [];

			var ind1 = " ";
			if(! _.isUndefined(field.ind1())) {
				ind1 = field.ind1();
			}

			var ind2 = " ";
			if(! _.isUndefined(field.ind2())) {
				ind2 = field.ind2();
			}
		

			tempField.push(field.name());
			tempField.push(ind1);
			tempField.push(ind2);
			tempField.push(new Array());

			_.each(field.get('subfields'), function(subField) {
				_.last(tempField).push([subField.name(), subField.value()]);
			});

			jsonHash.fields.push(tempField);

		}else {
			jsonHash.fields.push([field.name(), field.value()]);
		}	
	});

	this.modified(false);
	this.set('json', jsonHash);
	return this.get('json');
}

Record.prototype.marc = function() {
	if(arguments.length == 0) {
		return this.get('marc');
	}

	this.set('marc', arguments[0]);
	return this;
}

Record.prototype.toMarc = function() {
	
	if(! this.modified() && ! _.isNull(this.marc())) {
		return this.marc();
	}

	// Create the MARC format
	var fields = this.get('fields')
		, data = ''
		, values = '';

	_.each(fields, function(field) {

		var valuesLengthBegining = Buffer.byteLength(values);

		if(parseFloat(field.get('name')) > 8 ) {
			
			var ind1 = " ";
			if(! _.isUndefined(field.ind1())) {
				ind1 = field.ind1();
			}

			var ind2 = " ";
			if(! _.isUndefined(field.ind2())) {
				ind2 = field.ind2();
			}
			
			values += ind1;
			values += ind2;

			_.each(field.get('subfields'), function(subfield) {
				values += _us;
				values += subfield.name(); // Sub field name
				values += subfield.value(); // Sub field value
			});

			values += _rs;

		}else {
			values += field.value();
			values += _rs;
		}	

		var fieldValueLength = Buffer.byteLength(values) - valuesLengthBegining;

		var fieldData = field.name();
		fieldData += pad(4, fieldValueLength, '0');
		fieldData += pad(5, valuesLengthBegining, '0');
		data += fieldData;
	});
	
	// Assembling the MARC
	var marc = "";
	marc += this.leader()
	marc += data;
	marc += _rs;
	marc += values;
	marc += _gs;

	this.modified(false);
	this.marc(marc);
	return this.marc();
}

Record.prototype.addField = function() {

	var ret = null;

	if(arguments[0].length > 3 || parseFloat(arguments[0]) > 999) {
		return false;
	}

	if(arguments.length == 1) {
		// Data Field
		this.get('fields').push(new Field(arguments[0]));
		ret = _.last(this.get('fields'));

	}else if(arguments.length == 2) {
		// Control Field
		this.get('fields').push(new Field(arguments[0], arguments[1]));
		ret = this;
	}

	if(! _.isUndefined(this.get('preventorder'))
		&& this.get('preventorder')) {
		this._sortFields();
	}

	this.modified(true);
	return ret;
}

/**
 * deleteField()
 *
 * @param {Mixed} obj
 * @api public
 */
Record.prototype.deleteField = function(fieldNames) {

	var names = new Array();
	if(! _.isArray(fieldNames)) {
		names.push(parseFloat(fieldNames));
	}else {
		names = _.map(fieldNames, function(fieldName){ return parseFloat(fieldName); });
	}

	var newFields = _.filter(this.fields(), function(field){
		if(! _.include(names, parseFloat(field.name()))) return field;
	});

	this.set('fields', newFields);
	return this;
}

/**
 * deleteControlFields()
 *
 * @param {Mixed} obj
 * @api public
 */
Record.prototype.deleteControlFields = function() {
	var newFields = _.filter(this.fields(), function(field){
		if( parseFloat(field.name()) > 8 ) return field;
	});

	this.set('fields', newFields);
	return this;	
}

/**
 * deleteDataFields()
 *
 * @param {Mixed} obj
 * @api public
 */
Record.prototype.deleteDataFields = function() {
	var newFields = _.filter(this.fields(), function(field){
		if( parseFloat(field.name()) <= 8 ) return field;
	});

	this.set('fields', newFields);
	return this;	
}

/**
 * hasField()
 *
 * @param {Mixed} obj
 * @api public
 */
Record.prototype.hasField = function(fieldName) {

	if(_.isArray(fieldName)) {
		var result = new Array();
		var self = this;
		_.each(fieldName, function(name) {
			var field = _.find(self.fields(), function(field) {
				if( parseFloat(name) == parseFloat(field.name()) ) return field;
			});
			result.push( (! _.isUndefined(field)) );
		});
		return result;
	}else {
		var field = _.find(this.fields(), function(field) {
			if( parseFloat(fieldName) == parseFloat(field.name()) ) return field;
		});

		return  (! _.isUndefined(field));
	}
}

Record.prototype._sortFields = function() {
	this.set('fields', _.sortBy(this.fields(), function(field) {
		return parseFloat(field.name());
	}));
};


Record.prototype.recordlength = function() {
	
	if(! _.isNull(this.marc())) {
		return parseFloat(S(this.marc()).left(5).s);
	}

	var recordLength = 0;

	// Leader
	recordLength += 24;

	// Data Section
	var dataLength = (parseFloat(this.info().length_of_length_field())
		+ parseFloat(this.info().length_of_starting_char_position())
		+ 3) * _.size(this.fields()); // Data Section
	if(dataLength != 0) {
		recordLength += dataLength;
		recordLength += 1; // Ending RS after Data Section
	}
	
	// Values Section
	_.each(this.fields(), function(field) {
		if(field.isControlField()) {
			recordLength += Buffer.byteLength(field.value());
		}else {
			var fieldLength = 0;
			fieldLength += 2; // Indicators
			_.each(field.subfields(), function(subfield) {
				fieldLength += Buffer.byteLength(subfield.name());
				fieldLength += Buffer.byteLength(subfield.value());
				fieldLength += 1; // US between subfields
			});

			recordLength += fieldLength;
		}

		recordLength += 1; // Ending RS for each field
	});

	recordLength += 1; // Ending GS

	return recordLength;
};

Record.prototype.baseaddress = function() {

	if(! _.isNull(this.marc())) {
		return parseFloat((new Buffer(this.marc())).slice(12, 17).toString());
	}
		
	return (parseFloat(this.info().length_of_length_field())
		+ parseFloat(this.info().length_of_starting_char_position())
		+ 3)
	* _.size(this.fields())
	+ 25;
};

Record.prototype.leader = function() {

	/*
	if(! _.isNull(this.marc())) {
		return S(this.marc()).left(24).s;
	}
	*/

	return 	this.get('leader')
			.recordLength(this.recordlength())
			.baseAddress(this.baseaddress())
			.recordinfo(this.info())
			.create();
};

module.exports = exports = Record;