var should = require('should')
	, fs = require('fs')
	, _ = require('underscore');
	

describe('MARC Delete Field', function(){

	var Marc = require('../index');
	var record = null;

	before(function(done){
		done();
	});

	beforeEach(function() {
		record = new Marc.Record();
		record.addField(1, "200045015");
		record.addField(3, "Milas");
		record.addField(5, "20120704144521.0");
		record.addField(8, "120704b        tu            000 0      ");
		record.addField(20).subfield('a', "9759484420");
		record.addField(40).subfield('a', "TBET");
		record.addField(41).subfield('a', "tur");
		record.addField(50)
				.ind1('0')
				.ind2('0')
				.subfield('a', "DR435.A7")
				.subfield('b', ".G86");
		record.addField(90).subfield('a', "DR435.A7 .G86");
		record.addField(100).ind1('1').subfield('a', "Güner,Agah Oktay.");
		record.addField(245)
				.ind1('1')
				.ind2('0')
				.subfield('a', "Türkiye'nin jeopolitiği ve Ermeni Propagandası /")
				.subfield('c', "Agah Oktay Güner.");
		record.addField(260)
				.subfield('a', "[s.l]:")
				.subfield('b', "Türkiye Ekonomi ve Sosyal Araştırma Vakfı,")
				.subfield('c', "[n.d.]");
		record.addField(300)
				.subfield('a', "104 p.;")
				.subfield('c', "19 cm.");
		record.addField(504).subfield('a', "Includes bibliographical references (p.103-104)");
		record.addField(650).ind2('0').subfield('a', "Ermeni Sorunu.");
		record.addField(650).ind2('0').subfield('a', "Armenian question");
		record.addField(650)
				.ind2('0')
				.subfield('a', "Ermeniler")
				.subfield('x', "Tarih")
				.subfield('z', "Türkiye");
		record.addField(650)
				.ind2('0')
				.subfield('a', "Armenians")
				.subfield('x', "History")
				.subfield('z', "Turkey");
		record.addField(710).subfield('a', "Türkiye Ekonomi ve Sosyal Araştırmalar Vakfı.");
		record.addField(999).subfield('a', "eu 04.06.2012");
	});

	after(function(done) {
		done();
	});

	afterEach(function(done) {
		done();
	});

	it('should delete control field, if it exists', function() {
		record.deleteField(1);
		record.deleteField(3);
		var field001 = record.field(1);
		var field003 = record.field(3);
		field001.should.be.false;
		field003.should.be.false;
	});

	it('delete data field', function() {
		record.deleteField(245);
		record.deleteField(50);
		var field245 = record.field(245);
		var field50 = record.field(50);
		field245.should.be.false;
		field50.should.be.false;
	});

	it('should delete array of control fieldNames', function() {
		record.deleteField([1, 3, '5']);
		record.field(1).should.be.false;
		record.field(3).should.be.false;
		record.field(5).should.be.false;
	});

	it('should delete array of data fieldNames', function() {
		record.deleteField([245, '50', '650']);
		record.field(245).should.be.false;
		record.field(50).should.be.false;
		record.field(650).should.be.false;
	});

	it('should ignore, if we try to delete nonexist field', function() {
		record.deleteField([200, '111', '245']);
		record.field(200).should.be.false;
		record.field(111).should.be.false;
		record.field(245).should.be.exist;
	});

	it('should delete subfields, if it is data field', function() {
		record.field(245).deleteSub('a');
		record.field(245).deleteSub('b');
		record.field(245).subfield('a').should.be.empty;
		record.field(245).subfield('b').should.be.empty;
	});

	it('should delete control fields', function() {
		record.deleteControlFields();
		for(var i=1; i <= 8; i++) {
			record.field(i).should.be.false;
		}
	});

	it('should delete data fields', function() {
		record.deleteDataFields();
		for(var i = 9; i <= 999; i++) {
			record.field(i).should.be.false;
		}
	});

	it('hasField', function() {
		record.hasField(1).should.not.be.false;
		record.hasField(111).should.be.false;

		record.hasField([1, '3', 5, '8']).should.be.an.instanceOf(Array);
		(record.hasField([1, '3', 5, '8']))[0].should.be.true;
		(record.hasField([1, '3', 5, '8']))[1].should.be.true;
		(record.hasField([1, '3', 5, '8']))[2].should.be.true;
		(record.hasField([1, '3', 5, '8']))[3].should.be.true;

		record.hasField([901, '3', 900, '8']).should.be.an.instanceOf(Array);
		(record.hasField([901, '3', 900, '8']))[0].should.be.false;
		(record.hasField([901, '3', 900, '8']))[1].should.be.true;
		(record.hasField([901, '3', 900, '8']))[2].should.be.false;
		(record.hasField([901, '3', 900, '8']))[3].should.be.true;
	});
});

