var assert = require("assert")
	, fs = require('fs')
	, Marc = require('../index')
	, _ = require('underscore')
	, util = require('util');

suite('Array', function() {

	setup(function(){});

	suite('New Marc', function(){

		test('should return control field value is it is added before', function(){
			
			var record = new Marc.Record();

			var controlField1 = [1, "001 value"]
				, controlField2 = [3, "003 value"];

			record.addField(controlField1[0], controlField1[1]);
			record.addField(controlField2[0], controlField2[1]);

			var controlField = record.field(controlField1[0]);

			assert.equal(controlField1[0], controlField.name());
			assert.equal(controlField1[1], controlField.value());

			controlField = record.field(controlField2[0]);
			assert.equal(controlField2[0], controlField.name());
			assert.equal(controlField2[1], controlField.value());
			assert.equal(true, controlField.isControlField());
		});

		test('should return data field values is it is added before', function(){
			
			var record = new Marc.Record();

			var field1 = [100, " ", " ", [ ['a', '100a value'], ['b', '100b value'] ]]
				, field2 = [245, " ", " ", [ ['a', '245a value'], ['b', '245b value'] ]];

			record
			.addField(field1[0])
			.ind1(field1[1])
			.ind2(field1[2])
			.subfield(field1[3][0][0], field1[3][0][1])
			.subfield(field1[3][1][0], field1[3][1][1]);

			record
			.addField(field2[0])
			.ind1(field2[1])
			.ind2(field2[2])
			.subfield(field2[3][0][0], field2[3][0][1])
			.subfield(field2[3][1][0], field2[3][1][1]);

			var field = record.field(field1[0]);
			assert.equal(field1[1], field.ind1());
			assert.equal(field1[2], field.ind2());
			assert.equal(_.size(field1[3]), _.size(field.subfields()));
			assert.equal(field1[3][0][0], _.first(field.subfields()).name());
			assert.equal(field1[3][0][1], _.first(field.subfields()).value());

			field = record.field(field2[0]);
			assert.equal(field2[1], field.ind1());
			assert.equal(field2[2], field.ind2());
			assert.equal(_.size(field2[3]), _.size(field.subfields()));
			assert.equal(field2[3][0][0], _.first(field.subfields()).name());
			assert.equal(field2[3][0][1], _.first(field.subfields()).value());
		});

		test('should ignore indicator1 if it is control field', function(){
			var record = new Marc.Record();
			var field1 = [1, "001 value"];
			record.addField(field1[0], field1[1]);
			var field = record.field(field1[0]);
			field.ind1('indicator 1');
			assert.equal(true, _.isUndefined(field.ind1()));
		});

		test('should ignore indicator2 if it is control field', function(){
			var record = new Marc.Record();
			var field1 = [1, "001 value"];
			record.addField(field1[0], field1[1]);
			var field = record.field(field1[0]);
			field.ind2('indicator 2');
			assert.equal(true, _.isUndefined(field.ind2()));
		});

		test('should return false if fieldname longer than 3 chars', function(){
			
			var record = new Marc.Record({
				preventorder: true
			});
			
			var field1 = [1000, "1000 value"];
			assert.equal(false, record.addField(field1[0], field1[1]));
			assert.equal(false, record.addField(field1[0]));

			var field1 = ["1000", "1000 value"];
			assert.equal(false, record.addField(field1[0], field1[1]));
			assert.equal(false, record.addField(field1[0]));
		});

		test('should prevent tag order if the option is provided', function(){
			
			var record = new Marc.Record({
				preventorder: true
			});

			var field1 = [3, "003 value"]
				, field2 = [1, "001 value"];

			record.addField(field1[0], field1[1]);
			record.addField(field2[0], field2[1]);

			var fields = record.fields();

			assert.equal(field2[0], fields[0].name());
			assert.equal(field2[1], fields[0].value());
			assert.equal(field1[0], fields[1].name());
			assert.equal(field1[1], fields[1].value());
		});

		test('should ignore order if the option is false', function(){
			
			var record = new Marc.Record({
				preventorder: false
			});

			var field1 = [3, "003 value"]
				, field2 = [1, "001 value"];

			record.addField(field1[0], field1[1]);
			record.addField(field2[0], field2[1]);

			var fields = record.fields();

			assert.equal(field1[0], fields[0].name());
			assert.equal(field1[1], fields[0].value());
			assert.equal(field2[0], fields[1].name());
			assert.equal(field2[1], fields[1].value());
		});

		test('leader should set the default codes correctly', function(){

			var defaults = {
				record_status: "n",
				type_of_record: "a",
				bibliographic_level: " ",
				type_of_control: " ",
				character_coding_scheme: " ",
				indicator_count: 2,
				subfield_code_count: 2,
				encoding_level: " ",
				descriptive_cataloging_form: " ",
				multipart_resource_record_level: " ",
				length_of_length_field: 4,
				length_of_starting_char_position: 5,
				length_of_implementation_defined_portion: 0
			};

			var record = new Marc.Record();

			assert.equal(defaults.record_status, record.info().record_status());
			assert.equal(defaults.type_of_record, record.info().type_of_record());
			assert.equal(defaults.bibliographic_level, record.info().bibliographic_level());
			assert.equal(defaults.type_of_control, record.info().type_of_control());
			assert.equal(defaults.character_coding_scheme, record.info().character_coding_scheme());
			assert.equal(defaults.indicator_count, record.info().indicator_count());
			assert.equal(defaults.subfield_code_count, record.info().subfield_code_count());
			assert.equal(defaults.encoding_level, record.info().encoding_level());
			assert.equal(defaults.descriptive_cataloging_form, record.info().descriptive_cataloging_form());
			assert.equal(defaults.multipart_resource_record_level, record.info().multipart_resource_record_level());
			assert.equal(defaults.length_of_length_field, record.info().length_of_length_field());
			assert.equal(defaults.length_of_starting_char_position, record.info().length_of_starting_char_position());
			assert.equal(defaults.length_of_implementation_defined_portion, record.info().length_of_implementation_defined_portion());
		});

		test('leader should set the codes correctly', function(){

			var cataloginfo = {
				record_status: "n",
				type_of_record: "a",
				bibliographic_level: " ",
				type_of_control: "#",
				character_coding_scheme: "#",
				indicator_count: 2,
				subfield_code_count: 2,
				encoding_level: "#",
				descriptive_cataloging_form: "#",
				multipart_resource_record_level: "#",
				length_of_length_field: 4,
				length_of_starting_char_position: 5,
				length_of_implementation_defined_portion: 0
			};

			var record = new Marc.Record();

			record.info()
				.record_status(cataloginfo.record_status)
				.type_of_record(cataloginfo.type_of_record)
				.bibliographic_level(cataloginfo.bibliographic_level)
				.type_of_control(cataloginfo.type_of_control)
				.character_coding_scheme(cataloginfo.character_coding_scheme)
				.indicator_count(cataloginfo.indicator_count)
				.subfield_code_count(cataloginfo.subfield_code_count)
				.encoding_level(cataloginfo.encoding_level)
				.descriptive_cataloging_form(cataloginfo.descriptive_cataloging_form)
				.multipart_resource_record_level(cataloginfo.multipart_resource_record_level)
				.length_of_length_field(cataloginfo.length_of_length_field)
				.length_of_starting_char_position(cataloginfo.length_of_starting_char_position)
				.length_of_implementation_defined_portion(cataloginfo.length_of_implementation_defined_portion);

			assert.equal(cataloginfo.record_status, record.info().record_status());
			assert.equal(cataloginfo.type_of_record, record.info().type_of_record());
			assert.equal(cataloginfo.bibliographic_level, record.info().bibliographic_level());
			assert.equal(cataloginfo.type_of_control, record.info().type_of_control());
			assert.equal(cataloginfo.character_coding_scheme, record.info().character_coding_scheme());
			assert.equal(cataloginfo.indicator_count, record.info().indicator_count());
			assert.equal(cataloginfo.subfield_code_count, record.info().subfield_code_count());
			assert.equal(cataloginfo.encoding_level, record.info().encoding_level());
			assert.equal(cataloginfo.descriptive_cataloging_form, record.info().descriptive_cataloging_form());
			assert.equal(cataloginfo.multipart_resource_record_level, record.info().multipart_resource_record_level());
			assert.equal(cataloginfo.length_of_length_field, record.info().length_of_length_field());
			assert.equal(cataloginfo.length_of_starting_char_position, record.info().length_of_starting_char_position());
			assert.equal(cataloginfo.length_of_implementation_defined_portion, record.info().length_of_implementation_defined_portion());
		});

		test('leader should be constructed correctly', function(){
			// Empty case
			var record = new Marc.Record();
			var leader = record.leader();
			assert.equal("00025na   2200025   4500", leader);
			assert.equal(24, leader.length);

			// Defaults Control Fields case
			record = new Marc.Record();

			record.addField(1, "001value");
			record.addField(3, "003value");
			
			leader = record.leader();
			assert.equal("00068na   2200049   4500", leader);
			assert.equal(24, leader.length);

			// Real case
			record = realCatalog1();
			leader = record.leader();

			assert.equal("01266cam a2200325 i 4500", leader);
			assert.equal(24, leader.length);
		});

		test('toMarc', function(){
			var record = realCatalog1();
			assert.equal(fs.readFileSync('./test/sources/loc.mrc').toString(), record.toMarc());
			record = realCatalog2();
			assert.equal(fs.readFileSync('./test/sources/2.marc').toString(), record.toMarc());
		});

		test('toJson', function(){});
	});
});

function realCatalog1() {
	var record = new Marc.Record();

	record.info()
		.record_status('c')
		.type_of_record('a')
		.bibliographic_level('m')
		.character_coding_scheme('a')
		.descriptive_cataloging_form('i');

	record.addField(1, "4191015");
	record.addField(5, "20120705141424.0");
	record.addField(8, "801202s1981    nju      b    001 0 eng  ");
	record.addField(35).subfield('9', "(DLC)   80039683");
	record.addField(906)
			.subfield('a', "7")
			.subfield('b', "cbc")
			.subfield('c', "orignew")
			.subfield('d', "u")
			.subfield('e', "ocip")
			.subfield('f', "19")
			.subfield('g', "y-gencatlg");
	record.addField(10).subfield('a', "   80039683 ");
	record.addField(20).subfield('a', "0691039666");
	record.addField(20).subfield('a', "0691101175 (pbk.)");
	record.addField(40)
			.subfield('a', "DLC")
			.subfield('c', "DLC")
			.subfield('d', "DLC");
	record.addField(41)
			.ind1('1')
			.subfield('a', "eng")
			.subfield('h', "ger");
	record.addField(43).subfield('a', "e-it---");
	record.addField(50)
			.ind1('0')
			.ind2('0')
			.subfield('a', "N6921.F7")
			.subfield('b', "W313");
	record.addField(82)
			.ind1('0')
			.ind2('0')
			.subfield('a', "709/.45/51")
			.subfield('2', "19");
	record.addField(100)
			.ind1('1')
			.subfield('a', "Wackernagel, Martin,")
			.subfield('d', "1881-1962.");
	record.addField(240)
			.ind1('1')
			.ind2('0')
			.subfield('a', "Lebensraum des Künstlers in der florentinischen Renaissance.")
			.subfield('l', "English");
	record.addField(245)
			.ind1('1')
			.ind2('4')
			.subfield('a', "The world of the Florentine Renaissance artist :")
			.subfield('b', "projects and patrons, workshop and art market /")
			.subfield('c', "Martin Wackernagel ; translated by Alison Luchs.");
	record.addField(260)
			.subfield('a', "Princeton, N.J. :")
			.subfield('b', "Princeton University Press,")
			.subfield('c', "c1981.");
	record.addField(300)
			.subfield('a', "xxx, 447 p. ;")
			.subfield('c', "24 cm.");
	record.addField(500)
			.subfield('a', "Translation of Der Lebensraum des Künstlers in der florentinischen Renaissance.");
	record.addField(504)
			.subfield('a', "Includes bibliographies and index.");
	record.addField(650)
			.ind2('0')
			.subfield('a', "Art, Italian")
			.subfield('z', "Italy")
			.subfield('z', "Florence.");
	record.addField(650)
			.ind2('0')
			.subfield('a', "Art, Renaissance")
			.subfield('z', "Italy")
			.subfield('z', "Florence.");
	record.addField(650)
			.ind2('0')
			.subfield('a', "Artists")
			.subfield('z', "Italy")
			.subfield('z', "Florence.");
	record.addField(650)
			.ind2('0')
			.subfield('a', "Art patronage")
			.subfield('z', "Italy")
			.subfield('z', "Florence.");
	record.addField(991)
			.subfield('b', "c-GenColl")
			.subfield('h', "N6921.F7")
			.subfield('i', "W313")
			.subfield('p', "00001191974")
			.subfield('t', "Copy 1")
			.subfield('w', "BOOKS");

	return record;
};

function realCatalog2() {
	var record = new Marc.Record();

	record.addField(1, "200045015");
	record.addField(3, "Milas");
	record.addField(5, "20120704144521.0");
	record.addField(8, "120704b        tu            000 0      ");
	record.addField(20).subfield('a', "9759484420");
	record.addField(40).subfield('a', "TBET");
	record.addField(41).subfield('a', "tur");
	record.addField(50)
			.ind1('0')
			.ind2('0')
			.subfield('a', "DR435.A7")
			.subfield('b', ".G86");
	record.addField(90).subfield('a', "DR435.A7 .G86");
	record.addField(100).ind1('1').subfield('a', "Güner,Agah Oktay.");
	record.addField(245)
			.ind1('1')
			.ind2('0')
			.subfield('a', "Türkiye'nin jeopolitiği ve Ermeni Propagandası /")
			.subfield('c', "Agah Oktay Güner.");
	record.addField(260)
			.subfield('a', "[s.l]:")
			.subfield('b', "Türkiye Ekonomi ve Sosyal Araştırma Vakfı,")
			.subfield('c', "[n.d.]");
	record.addField(300)
			.subfield('a', "104 p.;")
			.subfield('c', "19 cm.");
	record.addField(504).subfield('a', "Includes bibliographical references (p.103-104)");
	record.addField(650).ind2('0').subfield('a', "Ermeni Sorunu.");
	record.addField(650).ind2('0').subfield('a', "Armenian question");
	record.addField(650)
			.ind2('0')
			.subfield('a', "Ermeniler")
			.subfield('x', "Tarih")
			.subfield('z', "Türkiye");
	record.addField(650)
			.ind2('0')
			.subfield('a', "Armenians")
			.subfield('x', "History")
			.subfield('z', "Turkey");
	record.addField(710).subfield('a', "Türkiye Ekonomi ve Sosyal Araştırmalar Vakfı.");
	record.addField(999).subfield('a', "eu 04.06.2012");

	return record;
};
