var assert = require("assert")
	, fs = require('fs')
	, Marc = require('../index')
	, _ = require('underscore');

suite('Array', function() {

	setup(function(){});

	suite('Marc.Writer', function() {

		test('should write record object to file', function() {
			
			var catalog1 = realCatalog1()
				, catalog2 = realCatalog2();

			var writer = Marc.Writer;

			writer.file('./test/output/out.mrc');
			writer.sources(catalog1).saveAsMarc();

			assert.equal(true, fs.existsSync('./test/output/out.mrc'));
			assert.equal(parseFloat(catalog1.recordlength()), fs.statSync('./test/output/out.mrc').size);
			assert.equal(fs.readFileSync('./test/sources/loc.mrc').toString(), fs.readFileSync('./test/output/out.mrc').toString());

			writer.file('./test/output/out2.mrc');
			writer.sources(catalog2).saveAsMarc();

			assert.equal(true, fs.existsSync('./test/output/out2.mrc'));
			assert.equal(parseFloat(catalog2.recordlength()), fs.statSync('./test/output/out2.mrc').size);
			assert.equal(fs.readFileSync('./test/sources/2.marc').toString(), fs.readFileSync('./test/output/out2.mrc').toString());
		});

		test('should write reader object to file', function() {
			
			var reader = Marc.Reader;
			var record1 = reader.read('./test/sources/loc.mrc');
			var record2 = reader.read('./test/sources/2.marc');
			
			var writer = Marc.Writer;

			writer.file('./test/output/loc.mrc');
			writer.sources(record1).saveAsMarc();

			assert.equal(true, fs.existsSync('./test/output/loc.mrc'));
			assert.equal(parseFloat(record1.recordlength()), fs.statSync('./test/output/loc.mrc').size);
			assert.equal(fs.readFileSync('./test/sources/loc.mrc').toString(), fs.readFileSync('./test/output/loc.mrc').toString());

			writer.file('./test/output/2.marc');
			writer.sources(record2).saveAsMarc();

			assert.equal(true, fs.existsSync('./test/output/2.marc'));
			assert.equal(parseFloat(record2.recordlength()), fs.statSync('./test/output/2.marc').size);
			assert.equal(fs.readFileSync('./test/sources/2.marc').toString(), fs.readFileSync('./test/output/2.marc').toString());
		});
	});
});

function realCatalog1() {
	var record = new Marc.Record();

	record.info()
		.record_status('c')
		.type_of_record('a')
		.bibliographic_level('m')
		.character_coding_scheme('a')
		.descriptive_cataloging_form('i');

	record.addField(1, "4191015");
	record.addField(5, "20120705141424.0");
	record.addField(8, "801202s1981    nju      b    001 0 eng  ");
	record.addField(35).subfield('9', "(DLC)   80039683");
	record.addField(906)
			.subfield('a', "7")
			.subfield('b', "cbc")
			.subfield('c', "orignew")
			.subfield('d', "u")
			.subfield('e', "ocip")
			.subfield('f', "19")
			.subfield('g', "y-gencatlg");
	record.addField(10).subfield('a', "   80039683 ");
	record.addField(20).subfield('a', "0691039666");
	record.addField(20).subfield('a', "0691101175 (pbk.)");
	record.addField(40)
			.subfield('a', "DLC")
			.subfield('c', "DLC")
			.subfield('d', "DLC");
	record.addField(41)
			.ind1('1')
			.subfield('a', "eng")
			.subfield('h', "ger");
	record.addField(43).subfield('a', "e-it---");
	record.addField(50)
			.ind1('0')
			.ind2('0')
			.subfield('a', "N6921.F7")
			.subfield('b', "W313");
	record.addField(82)
			.ind1('0')
			.ind2('0')
			.subfield('a', "709/.45/51")
			.subfield('2', "19");
	record.addField(100)
			.ind1('1')
			.subfield('a', "Wackernagel, Martin,")
			.subfield('d', "1881-1962.");
	record.addField(240)
			.ind1('1')
			.ind2('0')
			.subfield('a', "Lebensraum des Künstlers in der florentinischen Renaissance.")
			.subfield('l', "English");
	record.addField(245)
			.ind1('1')
			.ind2('4')
			.subfield('a', "The world of the Florentine Renaissance artist :")
			.subfield('b', "projects and patrons, workshop and art market /")
			.subfield('c', "Martin Wackernagel ; translated by Alison Luchs.");
	record.addField(260)
			.subfield('a', "Princeton, N.J. :")
			.subfield('b', "Princeton University Press,")
			.subfield('c', "c1981.");
	record.addField(300)
			.subfield('a', "xxx, 447 p. ;")
			.subfield('c', "24 cm.");
	record.addField(500)
			.subfield('a', "Translation of Der Lebensraum des Künstlers in der florentinischen Renaissance.");
	record.addField(504)
			.subfield('a', "Includes bibliographies and index.");
	record.addField(650)
			.ind2('0')
			.subfield('a', "Art, Italian")
			.subfield('z', "Italy")
			.subfield('z', "Florence.");
	record.addField(650)
			.ind2('0')
			.subfield('a', "Art, Renaissance")
			.subfield('z', "Italy")
			.subfield('z', "Florence.");
	record.addField(650)
			.ind2('0')
			.subfield('a', "Artists")
			.subfield('z', "Italy")
			.subfield('z', "Florence.");
	record.addField(650)
			.ind2('0')
			.subfield('a', "Art patronage")
			.subfield('z', "Italy")
			.subfield('z', "Florence.");
	record.addField(991)
			.subfield('b', "c-GenColl")
			.subfield('h', "N6921.F7")
			.subfield('i', "W313")
			.subfield('p', "00001191974")
			.subfield('t', "Copy 1")
			.subfield('w', "BOOKS");

	return record;
};

function realCatalog2() {
	var record = new Marc.Record();

	record.addField(1, "200045015");
	record.addField(3, "Milas");
	record.addField(5, "20120704144521.0");
	record.addField(8, "120704b        tu            000 0      ");
	record.addField(20).subfield('a', "9759484420");
	record.addField(40).subfield('a', "TBET");
	record.addField(41).subfield('a', "tur");
	record.addField(50)
			.ind1('0')
			.ind2('0')
			.subfield('a', "DR435.A7")
			.subfield('b', ".G86");
	record.addField(90).subfield('a', "DR435.A7 .G86");
	record.addField(100).ind1('1').subfield('a', "Güner,Agah Oktay.");
	record.addField(245)
			.ind1('1')
			.ind2('0')
			.subfield('a', "Türkiye'nin jeopolitiği ve Ermeni Propagandası /")
			.subfield('c', "Agah Oktay Güner.");
	record.addField(260)
			.subfield('a', "[s.l]:")
			.subfield('b', "Türkiye Ekonomi ve Sosyal Araştırma Vakfı,")
			.subfield('c', "[n.d.]");
	record.addField(300)
			.subfield('a', "104 p.;")
			.subfield('c', "19 cm.");
	record.addField(504).subfield('a', "Includes bibliographical references (p.103-104)");
	record.addField(650).ind2('0').subfield('a', "Ermeni Sorunu.");
	record.addField(650).ind2('0').subfield('a', "Armenian question");
	record.addField(650)
			.ind2('0')
			.subfield('a', "Ermeniler")
			.subfield('x', "Tarih")
			.subfield('z', "Türkiye");
	record.addField(650)
			.ind2('0')
			.subfield('a', "Armenians")
			.subfield('x', "History")
			.subfield('z', "Turkey");
	record.addField(710).subfield('a', "Türkiye Ekonomi ve Sosyal Araştırmalar Vakfı.");
	record.addField(999).subfield('a', "eu 04.06.2012");

	return record;
};